/*
  -------------------- file run.h
  Created 3 May 94 by Andrew Shapira, Rensselaer Polytechnic Institute.

	Copyright (C) 1994 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*	This file should be included only by functions in the library.
  	It is not intended to be used by applications.
*/

#ifndef _030594_171652
#define _030594_171652

#include "../libinclude/ntup.h"
#include "tmask.h"

#define WORKFUNC_ARGS \
  const int,const int,const int,const struct tuple_struct * const, \
  const struct coord_struct * const, \
  const struct tmask_struct * const, const struct tmask_struct * const, \
  struct tuple_struct * const, int * const

/* ----- status codes from work function.  In the comments below,
   let p and q be the parameters to the work function.
*/

/* No tuple found that satisfies p,q.  Solutions may or not exist, but
   in any case, the search did not find a solution.
*/
#define WORKFUNC_NOTFOUND   0

/* No tuple exists that satisfies p,q.  Differs from WORKFUNC_NOTFOUND
   in that this implies the work algorithm has proved that no solutions
   exist.
*/
#define WORKFUNC_NONEEXIST  1

/* Tuple found that satisfies p,q.  The work function sets
   the p,q fields of the solution tuple to the requested p,q.
*/
#define WORKFUNC_P          2

/* Tuple found that satisfied (x,q), where x >= p and the function
   made an attempt to make x as large as possible.  Note: in this case
   the caller probably does not want to call the work function
   again with (p+1,q).  The work function sets the p,q fields of
   the solution tuple to x,q.

   WORKFUNC_PMAX does not distinguish between the situation where the
   search space was exhausted, and where it was not.
*/
#define WORKFUNC_PMAX       3

void report_input P(( const struct to_ntup_struct * const to ));
void report_back P(( FILE * const f, const int, const int ));
void ntup_run P(( const struct to_ntup_struct *, const int,
  struct from_ntup_struct *, void (*) P ((WORKFUNC_ARGS)) ));
void ntup_check_input P(( const struct to_ntup_struct *, int * ));
void ntup_check_input_back P(( const int,const int,const double,int * const ));
void lev_nodes_print P((
  FILE * const, const double * const, const int, const int, const int ));

#endif /* _030594_171652 */
